from optparse import OptionParser
from intspan import overlap
import os, sys, re
from tools import reverse_complement, seq_overlap
import pysam

class BAM:

    bam = None
    reads = None
    min_mapq = 1

    def __init__(self, bam_file, min_mapq=1, sample_type=None):
	self.source = bam_file
        self.bam = pysam.Samfile(bam_file, "rb")
        self.min_mapq = min_mapq
	self.sample_type = sample_type

    def confirm(self, target, pos=[], feature=None, allele=None, regions=None, seq=False, breakpoint=None, refseq=None):
        feature = feature.lower()
        reads = []

        if target and not 'chr' in self.bam.references[0]:
            target = target.replace('chr', '')
	    
	if target and target[0] == 'k' and '_' in target and not '_' in self.bam.references[0] and ':' in self.bam.references[0]:
	    target = target.replace('_', ':')

        if target and target not in self.bam.references:
            #sys.stderr.write("%s not in bam targets\n" % (target))
            if seq:
                return []
	    elif feature == 'snv':
		return 0, 0
            else:
                return 0
        
        if feature == 'snv' or feature == 'inv':
            reads = self.confirm_snv(target, pos[0], allele, seq=seq)
        elif feature == 'del':
            reads = self.confirm_del(target, pos, seq=seq, refseq=refseq)
	    
	#special case: confirm_ins only returns number
        elif feature == 'ins':
            reads = self.confirm_ins(target, pos[0], allele, refseq=refseq)
	    
        elif feature == 'breakpoint':
            if not regions:
                reads = self.confirm_breakpoint_contig(target, pos, seq=seq)
            else:
                reads = self.confirm_breakpoint_genomic(regions, seq=seq, breakpoint=breakpoint)
		
        return reads

    def confirm_perfect(self, target, pos, allele, strand, seq=False, min_from_end=0, expansion=0):
        '''
        confirm every base in the target is the same as allele
        '''
	
	if target and 'chr' in target and not 'chr' in self.bam.references[0]:
            target = target.replace('chr', '')
	
	#needed because pos may be extracted from text file
	pos[0] = int(pos[0])
	pos[1] = int(pos[1])
	
	pos[0] = max(pos[0], 1)
        num_reads = 0
        reads = []
	reads_dict = {}
	
	if expansion > 1:
	    i = 1;
	    seq = allele
	    while i < expansion:
		allele = allele + seq
		i = i+1
	
	#for insertion longer than read length
	coverage = {}
	for i in range(pos[0], pos[1]+1):
	    coverage[i] = 0
	
	shorter_than_read = False
        for read in self.bam.fetch(target, pos[0], pos[1]):
	    if len(allele) <= read.rlen:
		shorter_than_read = True
		if read.pos+1 <= pos[0] and read.pos+read.rlen >= pos[1]:
		    status = "unknown"
		    from_end = min(pos[0] - read.pos -1, read.pos + read.rlen - pos[1])
		    offset = pos[0]-read.pos-1
		    bases = read.seq[offset:offset+len(allele)]
		
		    #only needs to rc bases if contig is aligned to negative strand, as read-sequence(reported) is same as contig sequence    
		    if strand == '-':
			bases = reverse_complement(bases)
		
		    if from_end < min_from_end:
			status = "fail_edge"
		    
		    elif bases.lower() == allele.lower():
			status = "perfect"
			if seq:
			    reads.append(read)
			num_reads += 1
		    
		    else:
			status = "non-identical_base"
		    			
		    #print "status:%s target:%s pos:%d-%d strand:%s %s read_start:%d allele:%s bases:%s seq:%s from-end:%d" % (status, target, pos[0], pos[1], strand, read.qname, read.pos, allele, bases, read.seq, from_end)   
	    
	#none of the reads is longer than the allele(insertion), or no read within pos
	if not shorter_than_read:
	    allele_seq = allele
	    if strand == '-':
		allele_seq = reverse_complement(allele)
		
	    for pileupcolumn in self.bam.pileup(target, pos[0]-1, pos[1]):
		if shorter_than_read:
		    break
		
		if pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
		    cov = pileupcolumn.n
		    
		    for pileupread in pileupcolumn.pileups:
			#allele not really longer than read length
			if len(allele) <= len(pileupread.alignment.seq):
			    shorter_than_read = True
			    break
			    			
			#pysam bug?
			if pileupread.qpos >= len(pileupread.alignment.seq):
			    continue
						
			if pileupread.alignment.seq[pileupread.qpos].lower() != allele_seq[pileupcolumn.pos-pos[0]+1]:
			    cov = cov - 1
			else:
			    reads_dict[pileupread.alignment.qname] = pileupread.alignment
		    coverage[pileupcolumn.pos+1] = cov
		    			    
	    #print "coverage", coverage
	    
	    #determine if allele is entirely covered
	    gaps = False
	    for i in range(pos[0], pos[1]+1):
		if coverage[i] == 0:
		    gaps = True
		    break
		
	    if not gaps:
		if seq:
		    reads = reads_dict.values()
		else:
		    num_reads = len(reads_dict.keys())
	
        if seq:
            return reads
        else:
            return num_reads
                               

    def confirm_snv(self, target, pos, allele, min_from_end=8, seq=False):
        '''
        just get the start position of the snv allele
        the length of the snv can be > 1
        '''
        
	num_reads = 0
	total = 0
	reads = []

        if not allele or len(allele) < 1:
            return 0, 0
	
        for pileupcolumn in self.bam.pileup(target, pos, pos):
            if pileupcolumn.pos == pos-1:
                for pileupread in pileupcolumn.pileups:
                    bases = pileupread.alignment.seq[pileupread.qpos:pileupread.qpos + len(allele)]
                    
		    #calculate from end pos
		    read_len = pileupread.alignment.rlen
		    if read_len == 0:
			read_len = len(pileupread.alignment.seq)
			
		    from_end = pileupread.qpos
		    if read_len - pileupread.qpos < from_end:
			from_end = read_len - pileupread.qpos - 1
		    #print "test", pileupread.qname, pileupread.alignment.seq
		    #print 'snv in read %s = %s indel:%d is_del:%d is_head:%d is_tail:%d, pos:%d, len:%d, seq:%s, bases:%s, from_end:%d' % (pileupread.alignment.qname, pileupread.alignment.seq[pileupread.qpos], pileupread.indel, pileupread.is_del, pileupread.is_head, pileupread.is_tail, pileupread.qpos, pileupread.alignment.rlen, pileupread.alignment.seq, bases, from_end)

		    if from_end >= min_from_end:
			if bases.lower() == allele.lower():
			    if seq:
				reads.append(pileupread.alignment)
			    else:
				num_reads += 1
				
			total += 1

	if not seq:
	    return num_reads, total
        else:
	    return reads, total
	
    def confirm_del(self, target, pos, refseq=None, fragment_buffer=1000, seq=False):
	"""
	Will extract all reads containing the deletion
	If the deletion size is the same but has different coordinate in the bam file,
	will compare the sequence after deletion to see if the read contains the same event
	"""
	size = pos[1] - pos[0] + 1
		
	good_reads = []
	bigger_than_rlen = False
	for read in self.bam.fetch(target, pos[0], pos[1]):
	    #print read.qname, read.pos, read.cigar, read.seq, read.rlen
	    if int(read.mapq) < int(self.min_mapq):
                continue
	    	    
	    if pos[1] - pos[0] + 1 > read.rlen:
		bigger_than_rlen = True
		break

	    if read.pos+1 <= pos[0] and read.pos + self.aligned_len(read) >= pos[1]:
		align = self.parse_cigar(read)
		dels = [a for a in align if a[0] == 'del' and a[2] - a[1] + 1 == size]
		
		#if there is a deletion of the same size identified from the cigar string
		if dels:
		    expected_seq = refseq.GetSequence(target, read.pos + 1, pos[0] - 1) + refseq.GetSequence(target, pos[1] + 1, read.pos + read.rlen + size)
		    
		    for d in dels:
			#if the start and end of the deletion match, good
			if d[1] == pos[0] and d[2] == pos[1]:
			    good_reads.append(read)
			    break
			
			read_seq = refseq.GetSequence(target, read.pos + 1, d[1] - 1) + refseq.GetSequence(target, d[2] + 1, read.pos + read.rlen + size)
			if read_seq.lower() == expected_seq.lower():
			    good_reads.append(read)
			    break
			    
	if bigger_than_rlen:
	    genome_buffer = 2 * fragment_buffer
	    regions = [[target, pos[0] - genome_buffer, pos[0]], [target, pos[1], pos[1] + genome_buffer]]
	    breakpoint = [[target, pos[0]], [target, pos[1]]]
	                  
	    good_reads = self.confirm_breakpoint_genomic(regions, breakpoint=breakpoint, seq=True)
		
	if not seq:
	    return len(good_reads)
	else:
	    return good_reads
		
    def parse_cigar(self, read):
	"""
	Reports cigar string alignment in genomic coordinate
	For use in confirm_ins() and confirm_del()
	"""
	align = []
	
	end = read.pos
	for i in range(len(read.cigar)):				    
	    if read.cigar[i][0] == 0:
		align_type = 'match'		
		start = end + 1
		end = start + read.cigar[i][1] - 1
		align.append([align_type, start, end])
		
	    elif read.cigar[i][0] == 1:
		align_type = 'ins'		
		align.append([align_type, end, read.cigar[i][1]])
		
	    elif read.cigar[i][0] == 2:
		align_type = 'del'
		start = end + 1
		end = start + read.cigar[i][1] - 1	
		align.append([align_type, start, end])
	    
	return align
    
    def aligned_len(self, read):
	"""
	Returns aligned length from cigar string
	"""
	
	if read.cigar:
	    #don't add insertion length
	    lens = [a[1] for a in read.cigar if a[0] != 1]
	    return sum(lens)
	else:
	    return read.rlen
	
    
    def confirm_ins(self, target, pos, allele, refseq=None, fragment_buffer=1000, big_cutoff=50):
	"""
	Will extract all reads containing the given insertion
	If the insertion size is the same but has different coordinate in the bam file,
	will compare the sequence after insertion to see if the read contains the same event
	"""
        size = len(allele)
		
	good_reads = []
	num_reads_extracted = 0
	bigger_than_rlen = False
	for read in self.bam.fetch(target, pos, pos):
	    num_reads_extracted += 1
	    
	    if len(allele) > read.rlen:
		bigger_than_rlen = True
		break
	    
	    if int(read.mapq) < int(self.min_mapq):
                continue
	    
            if read.pos+1 <= pos and read.pos + read.rlen >= pos:
		align = self.parse_cigar(read)
		ins = [a for a in align if a[0] == 'ins' and a[2] == size]
		
		#if there is an insertion of the same size identified from the cigar string
		if ins:
		    expected_seq = refseq.GetSequence(target, read.pos + 1, pos) + allele + refseq.GetSequence(target, pos + 1, read.pos + read.rlen - size)
		    
		    for i in ins:			    
			read_ins_start = i[1] - read.pos
			read_allele = read.seq[read_ins_start:read_ins_start + i[2]]
			    
			#if position is the same and allele is the same, good
			if i[1] == pos and read_allele.lower() == allele.lower():
			    good_reads.append(read)
			    break
			
			read_seq = refseq.GetSequence(target, read.pos + 1, i[1]) + read_allele + refseq.GetSequence(target, i[1]+1, read.pos + read.rlen - size)
			    
			if read_seq.lower() == expected_seq.lower():
			    good_reads.append(read)
			    break
	
	if bigger_than_rlen or (num_reads_extracted == 0 and len(allele) > big_cutoff):
	    genome_buffer = 2 * fragment_buffer
	             
	    num_pairs_novel_upstream = self.confirm_novel((target, max(1, pos - genome_buffer), pos), allele, breakpoint=pos)
	    num_pairs_novel_downstream = self.confirm_novel((target, pos + 1, pos + 1 + genome_buffer), allele, breakpoint=pos + 1)
	    num_clipped_reads = self.confirm_clipped_reads((target, pos - genome_buffer, pos + genome_buffer), breakpoint=pos, novel_seq=allele)
	    
	    num_reads = num_clipped_reads + min(num_pairs_novel_upstream, num_pairs_novel_downstream) 
	    	    
	else:
	    num_reads = len(good_reads)
    
	return num_reads
	
    def confirm_novel(self, region, novel_seq, breakpoint, min_olap=10, max_unmapped=1000):
	'''
	Given novel sequence, find read-pairs that have one unmapped mate that overlaps (with minimum overlap) with novel sequence
	'''
	
	unmapped = {}
	count = 0
	for read in self.bam.fetch(region[0], region[1], region[2]):
	    if read.is_paired and read.mate_is_unmapped:
		count += 1
		unmapped[read.qname] = read
		
		if count == max_unmapped:
		    break
		

	mates = 0
	found_mates = 0
	good_reads = []
		
	if unmapped:
	    mate_seq = self.get_unmapped_mate_seq(region[0], unmapped.values())
	    
	    for read in unmapped.values():
		if (breakpoint == region[1] and not read.is_reverse) or \
		   (breakpoint == region[2] and read.is_reverse):
		    continue		
		
		if mate_seq.has_key(read.qname):
		    if read.pos > region[1]:
			from_end = read.pos - region[1] + 1
		    else:
			from_end = region[1] - read.pos + 1
			
		    mates += 1
		    
		    m = re.search(mate_seq[read.qname], novel_seq, re.IGNORECASE)
		    if m:
			found_mates += 1
			
		    else:
			m = re.search(reverse_complement(mate_seq[read.qname]), novel_seq, re.IGNORECASE)
			if m:
			    found_mates += 1
			    
			else:
			    olap = seq_overlap(mate_seq[read.qname], novel_seq)
			    if olap > min_olap:
				found_mates += 1
				
			    else:
				olap = seq_overlap(reverse_complement(mate_seq[read.qname]), novel_seq)
				if olap > min_olap:
				    found_mates += 1
				    			    
	return found_mates
	    
    def confirm_clipped_reads(self, region, breakpoint, novel_seq, min_clipped=5):
	num_clipped_reads = 0
	
	for read in self.bam.fetch(region[0], region[1], region[2]):
	    clipped_seq = self.get_clipped_seq(read, breakpoint)
	    
	    if clipped_seq and len(clipped_seq[1]) > min_clipped:
		clipped_seq_matched = False
		
		if clipped_seq[0] == 'start':
		    olap = seq_overlap(novel_seq, read.seq)
		    
		    if olap > len(clipped_seq):
			clipped_seq_matched = True
		    
		else:
		    olap = seq_overlap(read.seq, novel_seq)
		    
		    if olap > len(clipped_seq):
			clipped_seq_matched = True
			
		if clipped_seq_matched:
		    num_clipped_reads += 1
		
	return num_clipped_reads

    
    def get_unmapped_mate_seq(self, target, reads, max_reads=1000):
	'''
	Given a list of reads that don't have mates mapped, extract the sequences of their unmapped mates
	Must NOT be run within another loop of iteration of read extraction
	'''
	seq = {}
	for unmapped_read in reads:
	    for read in self.bam.fetch(target, max(1, unmapped_read.pos - unmapped_read.rlen), unmapped_read.pos + unmapped_read.rlen):
		if read.qname == unmapped_read.qname and read.is_read1 != unmapped_read.is_read1:
		    seq[read.qname] = read.seq
        
	return seq
    
    def is_soft_clipped(self, read, breakpoint):
	'''
	Given read, determine if it's soft-clipped from cigar
	'''
	if read.cigar and (read.cigar[0][0] == 4 or read.cigar[-1][0] == 4):
	    #soft-clipped at beginning
	    if read.cigar[0][0] == 4:
		start = read.pos - read.cigar[0][1]
	    else:
		start = read.pos
		
	    end = start + self.aligned_len(read)
	    
	    if breakpoint >= start and breakpoint <= end:
		return True
	    
	
	return False
    
    def get_clipped_seq(self, read, breakpoint):
	if read.cigar and (read.cigar[0][0] == 4 or read.cigar[-1][0] == 4):
	    if read.cigar[0][0] == 4:
		start = read.pos - read.cigar[0][1]
	    else:
		start = read.pos
		
	    end = start + self.aligned_len(read)
	    
	    #make sure soft-clipped reads overlap with breakpoint
	    if breakpoint >= start and breakpoint <= end:
		if read.cigar[0][0] == 4:
		    print read.qname, read.cigar[0][1], read.seq[:read.cigar[0][1]], read.seq
		    return ('start', read.seq[:read.cigar[0][1]])
		else:
		    print read.qname, read.cigar[-1][1], read.seq[-1 * read.cigar[-1][1]:], read.seq
		    return ('end', read.seq[-1 * read.cigar[-1][1]:])
	    
	return None

    def confirm_breakpoint_contig(self, target, pos, seq=False):
        num_reads = 0
        reads = []

        pos[0] = max(pos[0], 1)

        for read in self.bam.fetch(target, pos[0], pos[1]):
            if int(read.mapq) < int(self.min_mapq) or read.is_unmapped:
                continue

            if read.pos+1 <= pos[0] and read.pos + read.rlen >= pos[1]:
                num_reads += 1

                if seq:
                    reads.append(read)
            elif read.rlen < (pos[1] - pos[0] + 1):
                num_reads += 1

                if seq:
                    reads.append(read)

        if not seq:
            return num_reads
        else:
            return reads
	
	
    def confirm_breakpoint_genomic(self, regions, max_pairs=10000, breakpoint=None, seq=False):
        num_reads = 0

        region1 = regions[0]
        region2 = regions[1]

        #get rid of 'chr' if bam file's genome doesn't have chr
        if not 'chr' in self.bam.references[0] and 'chr' in regions[0][0]:
            regions[0][0] = regions[0][0].replace('chr', '')
            regions[1][0] = regions[1][0].replace('chr', '')
	    
	#make sure start > 0
	region1[1] = max(1, region1[1])
	region2[1] = max(1, region2[1])
	
        if not region1[0] in self.bam.references or not region2[0] in self.bam.references:
	    if seq:
		return []
	    else:
		return num_reads

        #don't do whole chromosome comparisons
        if int(region1[1]) == 0 and int(region1[2]) == 0:
	    if seq:
		return []
	    else:
		return num_reads
        if int(region2[1]) == 0 and int(region2[2]) == 0:
	    if seq:
		return []
	    else:
		return num_reads
	
	#return 0 if breakpoint not given
	if breakpoint == None:
	    if not seq:
		return num_reads
	    else:
		return []

	reads1 = {}
	reads1_pos = {}
	for read in self.bam.fetch(region1[0], region1[1], region1[2]):
	    if int(read.mapq) < int(self.min_mapq):
		continue
	    
	    alen = self.aligned_len(read)
	    if not overlap([read.pos + 1, read.pos + alen], region1[1:]):
		continue
	    
	    if not ((read.pos + alen - 1 < breakpoint[0][1] and not read.is_reverse) or (read.pos > breakpoint[0][1] and read.is_reverse)):
		continue
	    
	    reads1[read.qname] = read
	    reads1_pos[read.qname + ' ' + str(read.pos)] = True
	    
	reads2 = {}
	for read in self.bam.fetch(region2[0], region2[1], region2[2]):
	    if int(read.mapq) < int(self.min_mapq):
		continue

	    alen = self.aligned_len(read)
	    if not overlap([read.pos + 1, read.pos + alen], region2[1:]):
		continue
	    
	    if not ((read.pos + alen - 1 < breakpoint[1][1] and not read.is_reverse) or (read.pos > breakpoint[1][1] and read.is_reverse)):
		continue
		
	    #make sure reads2 is not capturing an identical read already captured by reads1
	    if not reads1_pos.has_key(read.qname + ' ' + str(read.pos)):
		reads2[read.qname] = read
	    
	#find real pairs
	pairs = {}
	for name in reads1.keys():
	    if reads2.has_key(name):
		pairs[name] = reads1[name], reads2[name]
		
	#calculate stacking
	stacks1 = {}
	stacks2 = {}
	for name, reads in pairs.iteritems():
	    if not stacks1.has_key(reads[0].pos):
		stacks1[reads[0].pos] = 0
	    stacks1[reads[0].pos] += 1
	    
	    if not stacks2.has_key(reads[1].pos):
		stacks2[reads[1].pos] = 0
	    stacks2[reads[1].pos] += 1
		
	num_reads = len(pairs.keys())
			    
	#rejects reads that are stacking
	if (len(stacks1.keys()) == 1 and stacks1[stacks1.keys()[0]] > 1) or (len(stacks2.keys()) == 1 and stacks2[stacks2.keys()[0]] > 1):
	    num_reads = num_reads * -1
	   
	if not seq:
	    return num_reads
	else:
	    return pairs.values()
    
    #try to expand exonic sequence only did not work
    def expand_region_for_introns(self, region, breakpoint):
	expanded_region = []
	
	region_size = int(region[2]) - int(region[1]) + 1
	exon_size = 0

	if region[1] == breakpoint:
	    increment = 1
	    base = region[1]
	    exon_start = region[1]
	else:
	    increment = -1
	    base = region[2]
	    exon_start = region[2]
	    
	print 'region', region, region_size, breakpoint, exon_start
	    
	prev_coverage = None
	prev_base = None
	is_intron = False
	initial = 0
	intron_depth = 10
	max_span = 10000
	extended_upper = 0
	intron_start = -1
	intron_size = 0
	max_intron_size = 500000
	changed_prev = False
	coverages = {}
	reverse = False
		
	while exon_size < region_size:
	    #entering intergenic region
	    if intron_size > max_intron_size:
		#print 'break intron', intron_size
		break
	    
	    if not coverages.has_key(base):
		del coverages
		coverages = {}
		#print 'piling up', base-region_size, base+region_size
		for pileupcolumn in self.bam.pileup(region[0], base - region_size, base + region_size):
		    coverages[pileupcolumn.pos + 1] = pileupcolumn.n
		    #print 'coverage at base %d %s = %s' % (base, pileupcolumn.pos , pileupcolumn.n)
		    
		for i in range(base - region_size, base + region_size+1):
		    if not coverages.has_key(i):
			coverages[i] = 0
		    
	    if coverages.has_key(base):
		coverage = coverages[base]
	    else:
		coverage = 0
	    
		
	    if prev_coverage == None:
		prev_coverage = coverage
		prev_base = base
		continue
	    
	    if coverage <= intron_depth:
		is_intron = True
	    elif coverage < prev_coverage / 5:
		is_intron = True
	    elif coverage > prev_coverage * 5:
		#previous bases are all introns, check if intron size exceeds limit
		if not is_intron and not reverse:
		    reverse = True
		    intron_size = abs(base - exon_start)
		    #print 'reverse', intron_size
		    if intron_size > max_intron_size:
			#print "stop intron size", intron_size
			break
		    
		    exon_size -= intron_size
		    
		exon_start = base
		intron_start = -1
		is_intron = False
		intron_size = 0
	    else:
		reverse = False
		
	    
	    if not is_intron:
		exon_size += 1
		    
		if exon_size == region_size:
		    break
		    
	    else:
		if intron_start > 0:
		    intron_size = abs(base - intron_start) + 1
		else:
		    intron_start = base
			
		if intron_size > max_intron_size:
		    #print "stop intron size", intron_size
		    break
		
	    if abs(base - prev_base) == 100:
		prev_base = base
		prev_coverage = coverage
		
	    base += increment		    
		    
	print 'exon', exon_size, base
	
	if exon_size > 0:
	    if increment > 0 and base > int(region[2]):
		print 'changed', [region[0], int(region[1]), base]
		return [region[0], int(region[1]), base]
	    elif increment < 0 and base < int(region[1]):
		print 'changed', [region[0], base, int(region[2])]
		return [region[0], base, int(region[2])]
	
	return region

    def coverage_simple(self, target, pos=[], total=False, seq=False):
        ''' not counting split reads '''
        
        if not 'chr' in self.bam.references[0] and 'chr' in target:
            target = target.replace('chr', '')

        if not target in self.bam.references:
            if seq:
                return []
            else:
                return 0

        coverage_profile = {}
        for i in range(max(0, pos[0]), pos[1]+1):
            coverage_profile[i] = 0

        reads = []
        num_reads = 0
        for read in self.bam.fetch(target, max(0, pos[0]), pos[1]):
            read_end = read.pos + read.rlen - 1
            if read.pos+1 <= pos[1] and read_end+1 >= pos[0]:
                #print "coverage",read.qname, target, pos, read.pos, read_end, read.rlen, read.seq, read.is_proper_pair, read.is_paired, read.mpos, read.mrnm
                num_reads += 1
                
                if seq:
                    reads.append(read)
                    
                for i in range(max(read.pos, pos[0]-1), min(read_end, pos[1]-1)+1):
                    coverage_profile[i+1] += 1

        #print "coverage", coverage_profile

        if not total:
            return "%.1f" % (sum(coverage_profile.values()) / len(coverage_profile.keys()))
        elif seq:
            return reads
        else:
            return num_reads

    def coverage_simple_pileup(self, target, pos=[], max_depth=10000):
        ''' use pileup, will count split reads '''
        
        if not 'chr' in self.bam.references[0] and 'chr' in target:
            target = target.replace('chr', '')
        
        if not target in self.bam.references:
            return None
        
        pair_used = {}
        coverage_profile = {}
        for i in range(pos[0],pos[1]+1):
            coverage_profile[i] = 0

        for pileupcolumn in self.bam.pileup(target, pos[0], pos[1]):
            #if pileupcolumn.pos > pos[1]-1:
            #    break
            if pileupcolumn.n >= max_depth:
                return None
            
            if pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
                coverage_profile[pileupcolumn.pos+1] = pileupcolumn.n

        return "%.1f" % (sum(coverage_profile.values()) / len(coverage_profile.keys()))
                
    def coverage(self, target, pos=[], read=True, fragment=False, profile=False, use_orphan=True, insert_size=0):
        if not 'chr' in self.bam.references[0] and 'chr' in target:
            target = target.replace('chr', '')
        
        if not target in self.bam.references:
            return None
        
        pair_used = {}
        coverage_profile = {}
        #print target, pos
        for i in range(pos[0],pos[1]+1):
            coverage_profile[i] = 0

        pileup_start = pos[0]
        pileup_end = pos[1]
        if insert_size and insert_size > pos[1]-pos[0]:
            pileup_start = pos[1] - insert_size
            pileup_end = pos[0] + insert_size
            if pileup_start < 0:
                pileup_start = 0
                
        #print pos
        #print pileup_start, pileup_end
        for pileupcolumn in self.bam.pileup(target, pileup_start, pileup_end):
            #print 'coverage at base %s = %s' % (pileupcolumn.pos , pileupcolumn.n)
            if pileupcolumn.pos >= pileup_start-1 and pileupcolumn.pos <= pileup_end-1:
                #print 'coverage at base %s = %s' % (pileupcolumn.pos , pileupcolumn.n)

                for pileupread in pileupcolumn.pileups:
                    if fragment:
                        if pileupread.alignment.is_proper_pair:
                            if not pair_used.has_key(pileupread.alignment.qname):
                                if pileupread.alignment.pos < pileupread.alignment.mpos:
                                    start = pileupread.alignment.pos+1
                                    end = pileupread.alignment.mpos+1+pileupread.alignment.rlen-1
                                else:
                                    end = pileupread.alignment.pos+1+pileupread.alignment.rlen-1
                                    start = pileupread.alignment.mpos+1
                                    
                                for i in range(start, end+1):
                                    if i >= pos[0] and i <= pos[1]:
                                        coverage_profile[i] += 1

                                orient = '+'
                                if pileupread.alignment.is_reverse:
                                    orient = '-'
                                mate_orient = '+'
                                if pileupread.alignment.mate_is_reverse:
                                    mate_orient = '-'
                                #print "pair %d name:%s orient:%s pos:%d mate_orient:%s mate_pos:%d start:%d end:%d" % (pileupcolumn.pos, pileupread.alignment.qname, orient, pileupread.alignment.pos+1, mate_orient,  pileupread.alignment.mpos+1, start, end)
                                pair_used[pileupread.alignment.qname] = True
                        elif use_orphan:
                            #print "single %d name:%s pos:%d" % (pileupcolumn.pos, pileupread.alignment.qname, pileupread.alignment.pos+1)
                            if pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
                                coverage_profile[pileupcolumn.pos+1] += 1
                    elif pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
                        if use_orphan:
                            #print pileupread.alignment.qname, pos, pileupread.alignment.pos+1, target
                            coverage_profile[pileupcolumn.pos+1] += 1
                        else:
                            if pileupread.alignment.is_proper_pair:
                                coverage_profile[pileupcolumn.pos+1] += 1
                            
        if profile:
            return coverage_profile
        else:
            return "%.1f" % (sum(coverage_profile.values()) / (pileup_end-pileup_start+1))


def get_reads(bam, target, pos=[], feature=None, allele=None):
    "get reads covering specified region"

    reads = []
    
    if feature.lower() == 'snv' and allele:
        for pileupcolumn in bam.pileup(target, pos[0], pos[0]):
            if pileupcolumn.pos == pos[0]-1:
                #print 'coverage at base %s = %s' % (pileupcolumn.pos , pileupcolumn.n)
                for pileupread in pileupcolumn.pileups:
                    read = {}

                    read['name'] = pileupread.alignment.qname
                    read['from_end'] = None
                    read_len = pileupread.alignment.rlen
                    if read_len == 0:
                        read_len = len(pileupread.alignment.seq)
                    read['from_end'] = pileupread.qpos
                    if read_len - pileupread.qpos < read['from_end']:
                            read['from_end'] = read_len - pileupread.qpos - 1
                    
                    if pileupread.alignment.seq[pileupread.qpos].lower() == allele.lower():
                        read['support'] = True
                    else:
                        read['support'] = False

                    reads.append(read)
                    
    elif feature.lower() == 'del':
        reads_dict = {}
        
        for pileupcolumn in bam.pileup(target, pos[0], pos[1]):
            if pileupcolumn.pos >= pos[0]-1 and pileupcolumn.pos <= pos[1]-1:
                #print 'coverage at base %s = %s' % (pileupcolumn.pos , pileupcolumn.n)
                for pileupread in pileupcolumn.pileups:
                    if reads_dict.has_key(pileupread.alignment.qname):
                        read = reads_dict[pileupread.alignment.qname]
                    else:
                        read = {}
                        read['name'] = pileupread.alignment.qname
                        read['support'] = True
                        
                        read_len = pileupread.alignment.rlen
                        if read_len == 0:
                            read_len = len(pileupread.alignment.seq)
                        from_end = pileupread.qpos
                        if read_len - pileupread.qpos < from_end:
                            from_end = read_len - pileupread.qpos - pileupread.indel
                        if not read.has_key('from_end') or from_end < read['from_end']:
                            read['from_end'] = from_end
                            

                        reads_dict[pileupread.alignment.qname] = read
                        
                    if pileupread.is_del < 1:
                        read['support'] = False

        reads = reads_dict.values()
            
                    

    elif feature.lower() == 'ins':
        for pileupcolumn in bam.pileup(target, pos[0], pos[0]):
            if pileupcolumn.pos == pos[0]-1:
                #print 'coverage at base %s = %s' % (pileupcolumn.pos , pileupcolumn.n)
                for pileupread in pileupcolumn.pileups:
                    read = {}
                    
                    read['name'] = pileupread.alignment.qname
                    read['from_end'] = None
                    read_len = pileupread.alignment.rlen
                    if read_len == 0:
                        read_len = len(pileupread.alignment.seq)
                    
                    if pileupread.indel > 0:
                        read['support'] = True
                        read['bases'] = pileupread.alignment.seq[pileupread.qpos+1:pileupread.qpos+1+pileupread.indel]
                        read['from_end'] = pileupread.qpos + 1
                        if read_len - pileupread.qpos < read['from_end']:
                            read['from_end'] = read_len - pileupread.qpos - pileupread.indel - 1
                    else:
                        read['support'] = False
                        read['bases'] = None

                    reads.append(read)


if __name__ == '__main__':
    usage = "Usage: %proj bam_file"

    parser = OptionParser(usage=usage)
    parser.add_option("-f", "--feature", dest="feature", help="feature: snv,ins,del,breakpoint")
    parser.add_option("-t", "--target", dest="target", help="target, chr or contig number")
    parser.add_option("-s", "--start", dest="start", help="start coordinate", type='int')
    parser.add_option("-e", "--end", dest="end", help="end coordinate", type='int')
    parser.add_option("-a", "--allele", dest="allele", help="allele: base(s) for snv or insertion")
    parser.add_option("-c", "--coverage", dest="coverage", help="coverage", action="store_true", default=False)
    parser.add_option("-r", "--region1", dest="region1", help="region1 (breakpoint)", nargs=3)
    parser.add_option("-R", "--region2", dest="region2", help="region2 (breakpoint)", nargs=3)
    
    (options, args) = parser.parse_args()

    bam_file = args[0]
    if len(args) == 1 and os.path.exists(bam_file):
        bam = BAM(bam_file)

        if options.coverage:
            coverage = bam.coverage(options.target, (options.start, options.end))

	else:
            regions = None
            if options.region1:
                regions = [options.region1, options.region2]
            reads = bam.confirm(options.target, (options.start, options.end), feature=options.feature, allele=options.allele, regions=regions)

            for i in range(len(reads)):
                read = reads[i]
                output = []
                for attr in sorted(read.keys()):
                    output.append("%s:%s" % (attr, read[attr]))
		    
    else:
        parser.error("cannot find given bam file")

        
