import os, sys, tempfile, ConfigParser, random, glob, re, subprocess
from shutil import copy
from optparse import OptionParser
import assembly, cfg
from shutil import copy

PACKAGE_DIR = "/".join(os.path.abspath(sys.argv[0]).split("/")[:-2])
PYTHON_PATH = "export PYTHONPATH=.:$PYTHONPATH:" + PACKAGE_DIR + ";"
PYTHON = 'python'
CONFIG_DIR = PACKAGE_DIR + "/configs"
if not os.path.isdir(CONFIG_DIR):
    sys.exit(0)

class ClusterAlign:
    "run alignments on the cluster"
    
    name = None
    #can be full path
    aligner = None
    inputs = []
    input_file = None
    target = None
    out_dir = None
    jobs_file = None
    #make top_dir an optional parameter?
    top_dir = None

    align_config = None
    cluster_config = None
    cluster_head = None
    
    submit_script = None

    def __init__(self, config_file, name, aligner, input_file=None, target=None, out_dir=None, assembly=None, top_dir=None, cluster=None, params=None):
        self.name = name
        self.aligner = aligner
        self.input_file = input_file
        self.target = target
        self.out_dir = out_dir
        
        self.params = params

        self.cluster_head = cluster

        self.config_file = config_file
        self.align_config = cfg.initialize_settings(config_file)
        
        if cluster:
            self.cluster_config = cfg.initialize_settings(config_file)
            sys.stdout.write("cluster used: %s %s\n" % (cluster, config_file))
            self.submit_script = PACKAGE_DIR + "/utilities/submitjobs.sh"
                        
        #set up directory
        if not out_dir:
            if top_dir:
                if not os.path.isdir(top_dir):
                    os.mkdir(top_dir)
                self.top_dir = top_dir + "/cluster"
            elif assembly:
                self.top_dir = assembly.loc + "/cluster"

            if self.top_dir:
                #create topdir if not exists
                if not os.path.exists(self.top_dir):
                    os.mkdir(self.top_dir)

                if not os.path.exists(self.top_dir + "/" + name):
                    os.mkdir(self.top_dir + "/" + name)
                    os.mkdir(self.top_dir + "/" + name + "/jobs")
                    os.mkdir(self.top_dir + "/" + name + "/input")
                    os.mkdir(self.top_dir + "/" + name + "/output")


    def prepare_input(self, queries_per_job, out_dir=None, contigs=[], input_dir=None):
        "prepare input fasta files"

        #if out_dir is not specifically given and is following file structure...
        if not out_dir and self.top_dir:
            out_dir = self.top_dir + "/" + self.name + "/input"
        
        #link to input dir if provided
        if input_dir:
            if os.path.islink(out_dir):
                os.remove(out_dir)
            elif os.path.isdir(out_dir):
                os.rmdir(out_dir)
            os.symlink(input_dir, out_dir)

            for input_file in glob.glob(os.path.join(input_dir,'*.fa')):
                self.inputs.append(input_file)

        else:
            if not os.path.isdir(out_dir):
                os.mkdir(out_dir)

            lines = []
            seq_count = 0
            file_count = 1
            files = []
            order_indices = range(len(contigs))
            random.shuffle(order_indices)
            count = 0
            for i in order_indices:
                lines.append(contigs[i].fasta())
                seq_count = seq_count + 1

                if seq_count == queries_per_job or count == len(contigs)-1:
                    out_file = out_dir + "/seq." + str(file_count) + ".fa"
                    out = open(out_file, 'w')
                    out.writelines(lines)
                    out.close()
                    self.inputs.append(out_file)
                
                    file_count = file_count + 1
                
                    #reset counter
                    seq_count = 0
                    lines = []

                count = count + 1

    def create_jobs(self, jobs_file=None, target=None, out_dir=None):
        "create batch file of cluster jobs"

        if not jobs_file and self.top_dir:
            self.jobs_file = self.top_dir + "/" + self.name + "/jobs/jobs"
        else:
            self.jobs_file = jobs_file
            
        if not out_dir and self.top_dir:
            out_dir = self.top_dir + "/" + self.name + "/output"
            if not os.path.exists(out_dir):
                os.mkdir(out_dir)

        #remove old file if it's there
        if os.path.exists(self.jobs_file):
            os.remove(self.jobs_file)

        target_file = cfg.get_value(self.cluster_config, 'genomes', target)
        
        lines = []
        if target_file:
            for fasta in self.inputs:
                setup = "%s/setup" % (PACKAGE_DIR)
                script = os.path.abspath(sys.argv[0])
                if self.cluster_head == 'genesis':
                    setup = setup.replace('projects', 'scratch')
                    script = script.replace('projects', 'scratch')
                    
                line = "source %s;python %s %s -a %s -n %s -f %s -t %s -o %s" % (setup, script, self.config_file, self.aligner, self.name, fasta, target_file, out_dir)
                if options.min_identity:
                    line += " -I " + options.min_identity
                    
                lines.append(line + "\n")


        #write batch jobs file
        print self.jobs_file
        if lines:
            out = open(self.jobs_file, 'w')
            out.writelines(lines)
            out.close()

        return True

            
    def submit(self, jobs_file=None, mem=None):
        "submit batch alignment jobs through mqsub"
        
        if not jobs_file:
            jobs_file = self.jobs_file

        jobs_dir = os.path.dirname(self.jobs_file)
        
        if os.path.exists(jobs_file) and os.path.exists(self.submit_script) and os.path.exists(jobs_dir):
            command = self.submit_script + " " + self.cluster_head + " mqsub " + jobs_dir + " " + jobs_file + " " + self.name + " " + mem + " all.q"
            print command
            os.system(command)
    
    def run(self):
        "run an alignment job on cluster node"

        if not self.input_file or not self.target or not self.out_dir or not self.aligner:
            return False

        #copy query to local directories using tempfile
        if os.path.exists(self.input_file):
            query = tempfile.NamedTemporaryFile()
            copy(self.input_file, query.name)

        #copy target to local, if target is a file (if target is not a file, should get target path from configs
        if not os.path.exists(self.target):
            #check if file already copied to /var/tmp
            target = tempfile.NamedTemporaryFile()
            copy(self.target, target.name)
            self.target = target.name

        #make sure output directory is there
        if not os.path.isdir(self.out_dir):
            os.mkdir(self.out_dir)
            
        args_template = cfg.get_value(self.align_config, 'commands', self.aligner)
        mem = cfg.get_value(self.align_config, 'command-memory', self.aligner)
        if args_template and mem:
            args = {}
            args['TARGET'] = self.target
            args['QUERY'] = query.name
            args['OUTPUT'] = self.out_dir + "/" + os.path.splitext(os.path.basename(self.input_file))[0]
    
            cmd = cfg.create_cmd(self.aligner, args_template, args, source=PACKAGE_DIR + '/setup', binary=True)
            print cmd
            
            if cmd and os.path.getsize(self.input_file) > 0 and os.path.getsize(self.target) > 0:
                os.system(cmd)

    def batch(self, queries_per_job, submit, k=None, input_dir=None, mem=None):
        "given a fasta files, split input and run alignment on cluster"

        contigs = None
        if not input_dir:
            ab = assembly.Assembly(None, k=k, fasta=self.input_file)
            contigs = ab.get_contigs(sequence=True)
            sys.stderr.write("contigs parsed ... %d" % (len(contigs)))
        
        if contigs or input_dir:
            self.prepare_input(queries_per_job=int(queries_per_job), contigs=contigs, input_dir=input_dir)
            success = self.create_jobs(target=self.target)

            if success:
                print "successfully created jobs"
                if submit:
                    self.submit(mem=mem)

    def rerun_jobno(self, job_log):
        #filename = job_log.split("/")[-1]
        #jobno = filename.split(".")[1]
        script_file = os.path.splitext(job_log)[0]
        pattern = re.compile('/input/seq.(\d+).fa ')
        script = open(script_file, 'r')
        for line in script:
            if line.find("/input/seq.") != -1:
                jobno = pattern.search(line).group(1)
        script.close()
        return jobno


    def rerun_input_name(self, jobno):
        input_file = self.top_dir + "/" + self.name + "/input/seq." + jobno + ".fa"
        return input_file


    def rerun(self, queries_per_job, submit, k=None, input_dir=None, mem=None):
        "assume inputs are done properly, re-run alignment on cluster only for failed jobs"

        self.inputs = []
        input_dir = self.top_dir + "/" + self.name + "/input"
        job_dir = self.top_dir + "/" + self.name + "/jobs"
        output_dir = self.top_dir + "/" + self.name + "/output"
        error = {}
        for job_log in glob.glob(os.path.join(job_dir,'*.e*')):
            jobno = self.rerun_jobno(job_log)
            if jobno not in error:
                error[jobno] = -1#initial value before checking
            if os.path.getsize(job_log) == 0:#empty log!
                if error[jobno] == -1:
                    error[jobno] = 1
            else:
                log = open(job_log, 'r')
                cur_error = 0
                for line in log:
                    l = line.lower()
                    if l.find("err") != -1 or l.find("fail")!= -1:#failed run
                        cur_error = 1
                log.close()
                if cur_error == 0:
                    error[jobno] = 0#good run!
                else:
                    if error[jobno] == -1:
                        error[jobno] = 1#need to rerun
        for job_input in glob.glob(os.path.join(input_dir,'*.fa')):
            filename = job_input.split("/")[-1]
            jobno = filename.split(".")[1]
            output_filename = os.path.join(output_dir, filename.replace(".fa",".psl"));
            if not os.path.exists(output_filename):
                error[jobno] = 1
            else:
                #cmd = "tail -n1 " + output_filename + " | tr -d \"\\n\" | tr \"\\t\" \"\\n\" | wc -l"
                #print cmd + "\n"
                p1 = subprocess.Popen(["tail", "-n1", output_filename], stdout=subprocess.PIPE)
                p2 = subprocess.Popen(["tr", "-d", "\"\\n\""], stdin=p1.stdout, stdout=subprocess.PIPE)
                p3 = subprocess.Popen(["tr", "\"\\t\"", "\"\\n\""], stdin=p2.stdout, stdout=subprocess.PIPE)
                p4 = subprocess.Popen(["wc", "-l"], stdin=p3.stdout, stdout=subprocess.PIPE)
                count_fields = int(p4.communicate()[0])
                if count_fields != 20:
                    #print output_filename + " has " + str(count_fields) + " tabs in last line\n"
                    error[jobno] = 1
        for jobno in error.keys():
            if error[jobno] == 1:
                input_file = self.rerun_input_name(jobno)
                self.inputs.append(input_file)
        if self.inputs:
            success = self.create_jobs(target=self.target)

            if success:
                print "successfully created jobs"
                if submit:
                    self.submit(mem=mem)

if __name__ == '__main__':
    usage = "Usage: config_file"
    
    parser = OptionParser()
    parser.add_option("-a", "--aligner", dest="aligner", help="blat|exonerate")
    parser.add_option("-n", "--name", dest="name", help="name of jobs")
    parser.add_option("-f", "--fasta", dest="fasta_file", help="fasta file of input")
    parser.add_option("-t", "--target", dest="target", help="database file")
    parser.add_option("-o", "--output", dest="out_dir", help="output directory of alignments")
    parser.add_option("-s", "--submit", dest="submit", help="submit jobs", action="store_true", default=False)
    #additional options for running batch jobs
    parser.add_option("-b", "--batch", dest="batch", help="batch submit jobs", action="store_true", default=False)
    parser.add_option("-q", "--queries_per_job", dest="queries_per_job", help="queries_per_job")
    parser.add_option("-k", "--k_value", dest="k", help="assembly k value")
    parser.add_option("-i", "--input", dest="input_dir", help="provides input files(dir)")
    parser.add_option("-m", "--mem", dest="mem", help="memory required, exonerate=12.2G, blat=5G(default)", default="5G")
    #option to re-run batch jobs, use this instead of "-b", then only those previousdly-failed will be run
    parser.add_option("-r", "--rerun", dest="rerun", help="re-run batch jobs that failed previously", action="store_true", default=False)
    parser.add_option("-c", "--cluster", dest="cluster", help="cluster head node")
    parser.add_option("-I", "--min_identity", dest="min_identity", help="minimum identity(for Blat only)")
    (options, args) = parser.parse_args()
        
    params = {}
    if options.min_identity:
        params['minIdentity'] = options.min_identity

    if options.batch:
        jobs = ClusterAlign(args[0], name=options.name, aligner=options.aligner, input_file=options.fasta_file, target=options.target, top_dir=options.out_dir, cluster=options.cluster, params=params)
        jobs.batch(queries_per_job=options.queries_per_job, submit=options.submit, k=options.k, input_dir=options.input_dir, mem=options.mem)
    else:
        if options.rerun:
            jobs = ClusterAlign(args[0], name=options.name, aligner=options.aligner, input_file=options.fasta_file, target=options.target, top_dir=options.out_dir, cluster=options.cluster, params=params)
            jobs.rerun(queries_per_job=options.queries_per_job, submit=options.submit, k=options.k, input_dir=options.input_dir, mem=options.mem)
        else:
            job = ClusterAlign(args[0], name=options.name, aligner=options.aligner, input_file=options.fasta_file, target=options.target, out_dir=options.out_dir, cluster=options.cluster, params=params)
            job.run()
    

    
