import subprocess, shlex
import re, os, sys, glob
from optparse import OptionParser
from shutil import copy
import cfg

#binaries
SAMTOOLS = 'samtools'
BOWTIE = 'bowtie'
BOWTIE_BUILD = 'bowtie-build'
BWA = 'bwa'
EXPORT2FQ = 'fq_all2std.pl export2std'
PYTHON = 'python'

PACKAGE_DIR = "/".join(os.path.abspath(__file__).split("/")[:-2])
QSEQ2FQ = PYTHON + " " + PACKAGE_DIR + "/utilities/qseq2fq.py"
BAM2FQ = PYTHON + " " + PACKAGE_DIR + "/utilities/bam2fq.py"

def parse_datapath(datapath_file):
    libs = {}

    libname_pattern = re.compile(r'^\[(\S+)\]')

    library = None

    for line in open(datapath_file, 'r'):
        line = line.rstrip()
        libname = libname_pattern.search(line)

        if libname:
            library = libname.group(1)
        else:
            cols = line.split("\t")
            if len(cols) >= 2 and library:
                if not libs.has_key(library):
                    libs[library] = {}
                    libs[library]["lanes"] = {}
                if cols[0] == "meta_maq":
                    libs[library]["meta_maq"] = cols[1]
                elif cols[0] == "meta_bwa":
                    libs[library]["meta_bwa"] = cols[1]
                else:
                    #libs[library]["lanes"] = {}
                    libs[library]["lanes"][cols[0]] = cols[1]
            
            else:
                library = None

    return libs


#def run_commands(out_dir, cmds, shell_cmds=[]):
    #cmd_log = out_dir + "log"
    #cmd_err = out_dir + "err"
    #if os.path.exists(cmd_log):
        #os.system("rm -f " + cmd_log)
    #if os.path.exists(cmd_err):
        #os.system("rm -f " + cmd_err)
    #cmd_log_file = open(cmd_log, "a")
    #cmd_err_file = open(cmd_err, "a")
    #num = len(cmds)
    #for i in range(num):
        #cmd = cmds[i]
        #print >> cmd_log_file, cmd
        ##os.system(cmd)
        #args = shlex.split(cmd)
        #p = subprocess.Popen(args, stdout=cmd_log_file, stderr=cmd_err_file)
        #code = p.wait()
        #if (code != 0): #for some reason, subprocess fail, try it again in shell
            #print >> cmd_err_file, cmd + " error: returncode " + str(code)
            #while i < num:
                #shell_cmds.append(cmds[i])
                #i += 1
            #break
    #for cmd in shell_cmds:
        #print >> cmd_log_file, cmd
        #os.system(cmd)
    #cmd_log_file.close()
    #cmd_err_file.close()


def prepare_reads_assembly(assembly_dir, outdir, lib):
    infiles = glob.glob(os.path.join(assembly_dir, '*in'))
    print 'infiles:', infiles

    reads_files = []

    for infile in infiles:
        for line in open(infile, 'r'):
            reads_file = line.rstrip('\n')           
            reads_files.append(reads_file)
                
    print reads_files
    
    if len(reads_files) == 0:
        sys.stdout.write("cannot find reads files\n")

    #if bam file, sort by read-name (for bwa in scrubbing to work properly)
    if len(reads_files) == 1 and os.path.splitext(reads_files[0])[1] == '.bam':
        name_sorted_bam = os.path.splitext(os.path.split(reads_files[0])[1])[0] + '.name_sorted.bam'
        write_pair_info([name_sorted_bam], outdir + '/reads.txt', single_bam=True)
        cmd = 'samtools sort -n -m 3000000000 %s %s/%s' % (reads_files[0], outdir, os.path.splitext(name_sorted_bam)[0])
        print cmd
        os.system(cmd)
        
    else:
        fqs = []
        for rf in reads_files:
            outname = construct_outname(rf, lib)
            fqs.append(outname + ".fq")
            convert_fq(rf, outdir + "/" + outname)
        
        if fqs:
            write_pair_info(fqs, outdir + "/reads.txt")
        
    if not reads_files:
        get_assembly_input(assembly_dir, outdir, lib)
    
def get_assembly_input(assembly_dir, outdir, lib):
    log_dir = assembly_dir + '/log'
    log_files = glob.glob(os.path.join(log_dir, '*'))
    
    inputs = []
    num_bams = 0
    
    if log_files:
        log_file = sorted(log_files)[0]
        
        for line in open(log_file, 'r'):
            if line[:5] == 'ABYSS':
                fields = line.rstrip('\n').split()
                for field in fields:
                    if os.path.exists(field):
                        inputs.append(field)
                        if os.path.splitext(field)[1] == '.bam':
                            num_bams += 1
                break
    
    if inputs:
        if len(inputs) == 1 and num_bams == 1:
            #write_pair_info(inputs, outdir + "/reads.txt", single_bam=True)
            name_sorted_bam = "%s.name_sorted" % (os.path.splitext(os.path.split(inputs[0])[-1])[0])
            write_pair_info([name_sorted_bam + '.bam'], outdir + "/reads.txt", single_bam=True)
            cmd = "source %s/setup;%s sort -n %s %s/%s" % (PACKAGE_DIR, SAMTOOLS, inputs[0], outdir, name_sorted_bam)
            print cmd
            os.system(cmd)
            #copy(inputs[0], outdir)
        elif num_bams > 0 and num_bams != len(inputs):
            sys.stdout.write("bam files (%d) do not match inputs (%d) : %s\n" % (num_bams, len(inputs), ','.join(inputs)))
        elif num_bams == 0:
            fqs = []
            for rf in inputs:
                outname = construct_outname(rf, lib)
                fqs.append(outname + ".fq")
                convert_fq(rf, outdir + "/" + outname)
        
            if fqs:
                write_pair_info(fqs, outdir + "/reads.txt")
        else:
            sys.stdout.write("mix of bams and non-bams : %s\n" % (','.join(inputs)))
    else:
        sys.stdout.write("cannot find input\n")
        
        
def write_pair_info(names, out_file, single_bam=False):
    if not single_bam and (len(names) % 2 != 0 or len(names) == 0):
        sys.stdout.write("uneven number of reads detected - no output file\n")
    
    else:
        print 'test',names, out_file
        out = open(out_file, 'w')
        for i in range(len(names)):
            out.write(names[i])
            if i%2 == 0:
                out.write(' ')
            else:
                out.write('\n')
        out.close()
            
def convert_fq(path, ext):
    cmds = []

    infile = ext
    outfile = ext + ".fq"
    unzipped = False
    
    if os.path.exists(outfile):
        return outfile
    
    if os.path.exists(path):
        if '.bz2' in path:
            cmds.append("bzip2 -dc " + path + " > " + infile)
        elif '.gz' in path:
            cmds.append("gzip -dc " + path + " > " + infile)
        else:
            unzipped = True
            infile = path
    
    else:
        if os.path.exists(path + '.bz2'):
            cmds.append("bzip2 -dc " + path + ".bz2 > " + infile)
        elif os.path.exists(path + '.gz'):
            cmds.append("gzip -dc " + path + ".gzip > " + infile)

    infile_ext = os.path.splitext(infile)[1]
    if infile_ext == '.bam':
        prefix = os.path.splitext(os.path.split(path)[1])[0]
        #get rid of "2_lanes" in bam file, mess up use of "_2" later in determining read pairs
        prefix = prefix.replace('_', '-')
        outdir = os.path.split(ext)[0]
        #bai = path + ".bai"
        ##no index file in source
        #if not os.path.exists(bai):
            #infile = ext + ".bam"
            #cmds.append("ln -s " + path + " " + infile)
            #cmds.append(SAMTOOLS + " index " + infile)
            #cmds.append(BAM2FQ + " " + infile + " " + prefix + " " + outdir)
        #else:
            #cmds.append(BAM2FQ + " " + path + " " + prefix + " " + outdir)
        
        PICARD_DIR = os.environ['PICARD_DIR']
        fq1 = '%s/%s_1.fq' % (outdir, prefix)
        fq2 = '%s/%s_2.fq' % (outdir, prefix)
        cmd = 'java -Xmx2g -jar %s/SamToFastq.jar INPUT=%s FASTQ=%s SECOND_END_FASTQ=%s INCLUDE_NON_PF_READS=true VALIDATION_STRINGENCY=SILENT' % (PICARD_DIR, path, fq1, fq2)
        cmds.append(cmd)
        
    elif 'qseq' in infile:
        cmds.append(QSEQ2FQ + " " + infile + " " + outfile)
    elif 'export' in infile:
        cmds.append(EXPORT2FQ + " " + infile + " > " + outfile)
    elif 'fastq' in infile or 'fq' in infile or 'FASTQ' in infile or 'FQ' in infile:
        if unzipped:
            cmds.append("ln -s " + infile + " " + outfile)
        else:
            cmds.append("mv " + infile + " " + outfile)
            
    for cmd in cmds:
        print cmd
        os.system(cmd)

    #only remove if it's an unzipped file
    if os.path.exists(infile) and os.path.dirname(infile) == os.path.dirname(outfile):
        print "remove " + infile
        os.remove(infile)


def construct_outname(path, lib):
    fname = os.path.split(os.path.splitext(path)[0])[1]
    paths = path.split('/')
    lane = None
    for i in range(len(paths)-1):
        if paths[i] == lib:
            lane = paths[i+1]
            break
    if lane:
        return '_'.join((lane, fname))
    else:
        return fname
    

def prepare_reads(library, lanes, outdir):
    cmds = []
    
    #find, copy, unzip, convert to fq
    for lane, path in lanes.iteritems():       
        exports = glob.glob(os.path.join(path, '*export*.txt'))
        exports_zipped = glob.glob(os.path.join(path, '*export*.gz'))
        exports_bzipped = glob.glob(os.path.join(path, '*export*.bz2'))
        qseqs_bzipped = glob.glob(os.path.join(path, '*qseq*.bz2'))
        qseqs_txt = glob.glob(os.path.join(path, '*qseq.txt'))
        fastqs_zipped = glob.glob(os.path.join(path, '*fastq*.gz'))
        number_p = re.compile('_(\d+)$')
        m = number_p.search(lane)
        lane_num = m.group(1)

        if exports and os.path.exists(exports[0]):
            for export in exports:
                export_unzipped = os.path.split(export)[1]
                export_fq = export_unzipped + ".fq"
                if not os.path.exists(outdir + "/" + export_fq):
                    cmds.append(EXPORT2FQ + " " + export + " > " + outdir + "/" + export_fq)
                
        elif exports_zipped and os.path.exists(exports_zipped[0]):
            for export in exports_zipped:
                export_zipped = os.path.split(export)[1]
                export_unzipped = outdir + "/" + export_zipped.replace(".gz", "")
                export_fq = export_unzipped + ".fq"
                if not os.path.exists(export_fq):
                    cmds.append("gzip -dc " + export + " > " + export_unzipped)
                    cmds.append(EXPORT2FQ + " " + export_unzipped + " > " + export_fq)
                    cmds.append("rm " + export_unzipped)

        elif exports_bzipped and os.path.exists(exports_bzipped[0]):
            for export in exports_bzipped:
                export_zipped = os.path.split(export)[1]
                export_unzipped = outdir + "/" + export_zipped.replace(".bz2", "")
                export_fq = export_unzipped + ".fq"
                if not os.path.exists(export_fq):
                    cmds.append("bzip2 -dc " + export + " > " + export_unzipped)
                    cmds.append(EXPORT2FQ + " " + export_unzipped + " > " + export_fq)
                    cmds.append("rm " + export_unzipped)

        elif qseqs_bzipped and os.path.exists(qseqs_bzipped[0]):
            for qseq in qseqs_bzipped:
                qseq_zipped =  os.path.split(qseq)[1]
                qseq_unzipped = outdir + "/" + qseq_zipped.replace(".bz2", "")
                qseq_fq = qseq_unzipped + ".fq"
                if not os.path.exists(qseq_fq):
                    cmds.append("bzip2 -dc " + qseq + " > " + qseq_unzipped)
                    cmds.append(QSEQ2FQ + " " + qseq_unzipped + " " + qseq_fq)
                    cmds.append("rm " + qseq_unzipped)
                    
        elif qseqs_txt and os.path.exists(qseqs_txt[0]):
            for qseq in qseqs_txt:
                qseq_fq = outdir + "/" + os.path.split(qseq)[1] + ".fq"
                cmds.append(QSEQ2FQ + " " + qseq + " " + qseq_fq)
                
        elif fastqs_zipped and os.path.exists(fastqs_zipped[0]):
            for fastq in fastqs_zipped:
                fastq_zipped = os.path.split(fastq)[1]
                fastq_unzipped = outdir + "/" + fastq_zipped.replace(".gz", "")
                fq = fastq_unzipped + ".fq"
                if not os.path.exists(fq):
                    cmds.append("gzip -dc " + fastq + " > " + fastq_unzipped)
                    cmds.append("mv " + fastq_unzipped + " " + fq)      

        elif os.path.isdir(path + "basecalls"):
            qseq_dir = path + "basecalls/qseq_summary"
            print qseq_dir
            if os.path.isdir(qseq_dir):
                for i in (1,2):
                    qseq_bz = "%s/s_%s_%d_concat_qseq.txt.bz2" % (qseq_dir, lane_num, i)
                    qseq_unzipped =  "%s/s_%s_%d_concat_qseq.txt" % (qseq_dir, lane_num, i)
                    
                    if (i==2):
                        qseq_bz3 = "%s/s_%s_%d_concat_qseq.txt.bz2" % (qseq_dir, lane_num, 3)
                        qseq_unzipped3 =  "%s/s_%s_%d_concat_qseq.txt" % (qseq_dir, lane_num, 3)
                        
                        if os.path.exists(qseq_bz3) or os.path.exists(qseq_unzipped3):
                            qseq_bz = qseq_bz3
                            qseq_unzipped = qseq_unzipped3
                            
                    qseq_file = "%s/%s_%d.qseq" % (outdir, lane, i)
                    qseq_fq = qseq_file + ".fq"
                    if not os.path.exists(qseq_fq):
                        if not os.path.exists(qseq_file):
                            if os.path.exists(qseq_bz):
                                cmds.append("bzip2 -dc " + qseq_bz + " > " + qseq_file)
                                cmds.append(QSEQ2FQ + " " + qseq_file + " " + qseq_fq)
                                cmds.append("rm " + qseq_file)
                            elif os.path.exists(qseq_unzipped):
                                cmds.append(QSEQ2FQ + " " + qseq_unzipped + " " + qseq_fq)

        else:
            sys.stderr.write("no reads found: " + lane + " " + path + "\n")

    for cmd in cmds:
        print cmd
        os.system(cmd)


def prepare_contigs_bwa(library, contigs_loc, outdir):
    contigs_outdir = outdir
    if not os.path.isdir(contigs_outdir):
        os.mkdir(contigs_outdir)
    
    contig_files = []

    if os.path.isdir(contigs_loc):
        contig_files = glob.glob(os.path.join(contigs_loc, '*.fa'))
        contigs_dest = contigs_outdir + "/" + library + "-contigs.fa"
    else:
        contig_files.append(contigs_loc)
        contigs_dest = contigs_outdir + "/" + os.path.split(contigs_loc)[1]

    out_fas = []
    for in_fa in contig_files:
        out_fa = contigs_outdir + "/" + os.path.split(in_fa)[1]
        if in_fa != out_fa:
            modify_contigs(in_fa, out_fa, library)
        out_fas.append(out_fa)

    if os.path.isdir(contigs_loc):
        tmp_out = contigs_outdir + "/tmp_contigs.fa"
        cmd = 'cat ' + ' '.join(out_fas) + ' > ' + tmp_out
        #print cmd
        os.system(cmd)

        for fa in out_fas:
            os.remove(fa)

        cmd = 'mv ' + tmp_out + ' ' + contigs_dest
        #print cmd
        os.system(cmd)

    
    cmd = BWA + " index " + contigs_dest

    print cmd
    os.system(cmd)

    cmd = SAMTOOLS + " faidx " + contigs_dest
    print cmd
    os.system(cmd)


def prepare_contigs_bowtie(library, contigs_loc, outdir):
    contig_files = []
    contigs_dest = None
    out_fas = []

    if os.path.isdir(contigs_loc):
        contig_files = glob.glob(os.path.join(contigs_loc, '*.fa'))
        contigs_dest = outdir + "/" + library + "-contigs.fa"
    else:
        contig_files.append(contigs_loc)
        contigs_dest = outdir + "/" + os.path.split(contigs_loc)[1]
    
    for in_fa in contig_files:
        out_fa = outdir + "/" + os.path.split(in_fa)[1]
        if in_fa != out_fa:
            modify_contigs(in_fa, out_fa, library)
        out_fas.append(out_fa)

    if os.path.isdir(contigs_loc):
        tmp_out = outdir + "/tmp_contigs.fa"
        cmd = 'cat ' + ' '.join(out_fas) + ' > ' + tmp_out
        #print cmd
        os.system(cmd)

        for fa in out_fas:
            os.remove(fa)

        cmd = 'mv ' + tmp_out + ' ' + contigs_dest
        #print cmd
        os.system(cmd)

    base = os.path.splitext(contigs_dest)[0]
    cmd = BOWTIE_BUILD + " " + contigs_dest + " " + base

    print cmd
    os.system(cmd)

    cmd = SAMTOOLS + " faidx " + contigs_dest
    print cmd
    os.system(cmd)

def modify_contigs(in_fa, out_fa, library, size_limit=10000):
    print 'modifying contigs', in_fa, out_fa
    out_lines = []
    seq = ''
    for line in open(in_fa, 'r'):
        line = line.rstrip('\n')
        if '>' in line:
            if len(seq) > 0:
                if len(seq) > size_limit:
                    seqs = split_len(seq, size_limit)
                    out_lines.extend(seqs)
                else:
                    out_lines.append(seq)
            name = line.split(' ')[0]
            new_name = re.sub('[+-]', '', name)
            new_name = re.sub('[:,]', '_', new_name)
            
            out_lines.append(line.replace(name, new_name))
            #out_lines.append(line)
            seq = ''
        elif re.search('[agtcAGTG]', line):
            seq += line
    if len(seq) > 0:
        if len(seq) > size_limit:
            seqs = split_len(seq, size_limit)
            out_lines.extend(seqs)
        else:
            out_lines.append(seq)

    out = open(out_fa, 'w')
    for line in out_lines:
        out.write(line + '\n')

def split_len(seq, length):
    return [seq[i:i+length] for i in range(0, len(seq), length)]

def get_fq(path):
    fqs = {}
    
    for fq in glob.glob(os.path.join(path, '*.fq')):
        filename = os.path.splitext(os.path.split(fq)[1])[0]
        prefix = filename
        if "_1" in filename:
            prefix = filename.split("_1")[0]
        elif "_2" in filename:
            prefix = filename.split("_2")[0]

        print prefix
        try:
            fqs[prefix].append(fq)
        except KeyError:
            fqs[prefix] = []
            fqs[prefix].append(fq)

    for prefix in fqs.keys():
        if len(fqs[prefix]) == 2:
            if os.stat(fqs[prefix][0]).st_size == 0 or os.stat(fqs[prefix][1]).st_size == 0:
                del fqs[prefix]
        else:
            if os.stat(fqs[prefix][0]).st_size == 0:
                del fqs[prefix]

    print fqs
    return fqs

def align_reads_bwa(library, contigs_file, dest):
    cmds = []
    shell_cmds = []
    
    reads_dir = dest
    contigs_dest = dest + "/" + os.path.split(contigs_file)[1]

    sorted_bams = []

    fqs = get_fq(reads_dir)
    for prefix in fqs.keys():
        sais = []
        for fq in fqs[prefix]:
            #align each end of each prefix
            sai = dest + "/" + os.path.split(fq)[1] + ".sai"
            cmds.append(BWA + " aln " + contigs_dest + " " + fq + " > " + sai)
            sais.append(sai)

        #put the pairs together
        if len(sais) == 2:
            sam = dest + "/" + prefix + ".sam"
            cmds.append(BWA + " sampe " + contigs_dest + " " + sais[0] + " " + sais[1] + " " + fqs[prefix][0] + " " + fqs[prefix][1] + " > " + sam)
                
            #covert sam to bam
            fai = dest + "/" + os.path.split(contigs_file)[1] + ".fai"
            bam = dest + "/" + prefix + ".bam"
            cmds.append(SAMTOOLS + " import " + fai + " " + sam + " " + bam)

            #skip sorting of individual bams
            #sort bam file
            ##bam_sorted = dest + "/" + prefix + ".sorted"
##            cmds.append(SAMTOOLS + " sort " + bam + " " + bam_sorted)
##            sorted_bams.append(bam_sorted + ".bam")

            sorted_bams.append(bam)

            #removing sam file and sai files
            #cmds.append("rm -f " + sam + " " + sais[0] + " " + sais[1] + " " + bam)

            #single end fq?
                
    #merge bam file
    merged_bam = dest + "/" + os.path.split(contigs_file)[1] + ".merged.bam"
    merged_sorted_bam = dest + "/" + os.path.split(contigs_file)[1] + ".bam"

    #merge and sort bam file
    print "test", fqs, len(fqs)/2, len(sorted_bams)
    if len(fqs) == len(sorted_bams):
        if len(sorted_bams) > 1:
            cmds.append(SAMTOOLS + " merge " + merged_bam + " " + " ".join(sorted_bams))
            cmds.append(SAMTOOLS + " sort -m 3000000000 " + merged_bam + " " +  dest + "/" + os.path.split(contigs_file)[1])
        else:
            cmds.append("mv " + sorted_bams[0] + " " + merged_sorted_bam)

        #index merged,sorted bam file
        cmds.append(SAMTOOLS + " index " + merged_sorted_bam)

    #remove individual bam files
    #shell_cmds.append("rm -f " + " ".join(sorted_bams))
    #shell_cmds.append("rm -f " + merged_bam)

    #remove contigs file
    #shell_cmds.append("rm -Rf " + dest + "/contigs")

    for cmd in cmds:
        print cmd
        os.system(cmd)
    #run_commands(dest + "/", cmds, shell_cmds)
    
def align_bwa_bam(lib, contigs_file, dest):
    cmds = []
    
    print lib, contigs_file, dest
    bam_files = glob.glob(os.path.join(dest + '/*.bam'))
    print bam_files
    
    if not bam_files:
        return None
    else:
        bam_file = bam_files[0]
    
    for i in (1,2):
        cmds.append('%s aln %s -b%d %s > %s.%d.sai' % (BWA, contigs_file, i, bam_file, bam_file, i))
                    
    cmds.append("%s sampe %s %s.1.sai %s.2.sai %s %s > %s/%s-contigs.sam" % (BWA, contigs_file, bam_file, bam_file, bam_file, bam_file, dest, lib))

    cmds.append("%s view -bS %s/%s-contigs.sam > %s/%s-contigs.unsorted.bam" % (SAMTOOLS, dest, lib, dest, lib))
    cmds.append("%s sort %s/%s.unsorted.bam %s/%s-contigs" % (SAMTOOLS, dest, lib, dest, lib))
    cmds.append("%s index %s/%s-contigs.bam" % (SAMTOOLS, dest, lib))
    
    for cmd in cmds:
        print cmd
        os.system(cmd)


def align_reads_bowtie(library, contigs_file, dest):
    "bowtie unpaired alignment of reads to contigs"
    
    cmds = []
    shell_cmds = []
    
    reads_dir = dest
    contigs_dest = dest
    
    final_bam = None
    
    fqs = [fq for fq in glob.glob(os.path.join(dest, '*.fq')) if os.stat(fq)[6] > 0]
    if fqs:
        fq_list = ','.join(fqs)

        ebwt = os.path.splitext(dest + "/" + os.path.split(contigs_file)[1])[0]
        unsorted_sam = ebwt + ".unsorted.sam"
        
        cmd = "%s %s %s -v 0 -a -p 8 -S %s --sam-nosq" % (BOWTIE, ebwt, fq_list, unsorted_sam)
        #cmd = "%s %s %s -v 0 -a -p 8 -S %s" % (BOWTIE, ebwt, fq_list, unsorted_sam)
        cmds.append(cmd)

        #convert sam to bam, sort
        unsorted_bam = ebwt + ".unsorted.bam"
        fai = dest + "/" + os.path.split(contigs_file)[1] + ".fai"
        cmds.append("%s view -bt %s -o %s %s" % (SAMTOOLS, fai, unsorted_bam, unsorted_sam))
        #cmds.append("%s view -bS %s >%s" % (SAMTOOLS, unsorted_sam, unsorted_bam))

        #sort bam file
        cmds.append("%s sort -m 3000000000 %s %s" % (SAMTOOLS, unsorted_bam, ebwt))

        #index bam file
        sorted_bam = ebwt + ".bam"
        final_bam = sorted_bam
        cmds.append("%s index %s" % (SAMTOOLS, sorted_bam))

        #remove files
        #shell_cmds.append("rm -f " + unsorted_bam)
        #cmds.append("rm -f " + unsorted_sam)

        #remove contig files
        #cmds.append("rm -f " + ebwt + "*.ebwt")
        #cmds.append("rm -f " + ebwt + "*.fai")
        cur_dir = os.path.dirname(unsorted_bam);

    #run_commands(dest + "/", cmds, shell_cmds)
    for cmd in cmds:
        print cmd
        os.system(cmd)
        
    #clean up
    if os.path.exists("final_bam" + ".bai"):
        files = ['*.ebwt', '*.fai', '*unsorted*', '*.fq']
        clean_up(dirname, files)
        
def align_reads_abyssmap(library, contigs_file, dest, k, bamfile=True):
    query = None
    if bamfile:
        bam_files = glob.glob(os.path.join(dest + '/*.bam'))
        if len(bam_files) > 0:
            query = bam_files[0]
    else:
        fqs = [fq for fq in glob.glob(os.path.join(dest, '*.fq')) if os.stat(fq)[6] > 0]
        query = ' '.join(fqs)
        
    cmds = []
    if query:
        cmds.append("source %s/setup;which abyss-map" % (PACKAGE_DIR))
        sam = "%s.abyss-map.sam" % (library)
        cmds.append("time abyss-map %s %s -j 8 -k %s > %s/%s" % (query, contigs_file, k, dest, sam))
        unsorted_bam = "%s.abyss-map.unsorted.bam" % (library)
        cmds.append("time samtools view -bS %s/%s -o %s/%s" % (dest, sam, dest, unsorted_bam))
        sorted_bam_prefix = "%s.abyss-map" % (library)
        cmds.append("time samtools sort -m 8000000000 %s/%s %s/%s" % (dest, unsorted_bam, dest, sorted_bam_prefix))
        cmds.append("time samtools index %s/%s.bam" % (dest, sorted_bam_prefix))
        
        for cmd in cmds:
            print cmd
            os.system(cmd)
            
def clean_up(dirname, files):
    all_files = []
    for ff in files:
        all_files.extend(glob.glob(os.path.join(dirname, ff)))
                         
    sys.stdout.write("will get rid of:\n" % (','.join(all_files)))
    for ff in all_files:
        os.remove(ff)
    


if __name__ == '__main__':
    usage = "Usage: %prog lib contigs(dir|file) dest"
    parser = OptionParser(usage=usage)
    parser.add_option("-i", "--info", dest="info", help="data path file", default="/archive/solexa1_4/analysis/data_path/illumina_data_paths.txt")
    parser.add_option("-r", "--reads", dest="reads", help="prepare reads in fq format (only action)",  action="store_true", default=False)
    parser.add_option("-c", "--contigs", dest="contigs", help="index contigs file", action="store_true", default=False)
    parser.add_option("-a", "--aligner", dest="aligner", help="bwa|bowtie|abyss-map", default="bwa")
    parser.add_option("-d", "--dir", dest="dir", help="assembly dir")
    parser.add_option("-b", "--bam", dest='bam', help="input is bam file", action="store_true", default=False)
    parser.add_option("-k", "--k", dest='k', help="kmer for abyss-map")

    (options, args) = parser.parse_args()

    if options.reads:
        if options.dir:
            #get_assembly_input(options.dir, args[2], lib=args[0])
            prepare_reads_assembly(options.dir, args[2], lib=args[0])
        else:
            libs = parse_datapath(options.info)
            if libs.has_key(args[0]):
                prepare_reads(args[0], libs[args[0]]['lanes'], args[2])
            else:
                os.stderr.write("%s is not in datapath file %s\n" % (args[0], options.info))

    elif options.contigs:
        if os.path.exists(args[1]):
            if options.aligner == 'bwa':
                prepare_contigs_bwa(args[0], args[1], args[2])
            elif options.aligner == 'bowtie':
                prepare_contigs_bowtie(args[0], args[1], args[2])

    else:
        if options.aligner == 'bwa':
            if not options.bam:
                align_reads_bwa(args[0], args[1], args[2])
            else:
                align_bwa_bam(args[0], args[1], args[2])
                
        elif options.aligner == 'bowtie':
            align_reads_bowtie(args[0], args[1], args[2])

        elif options.aligner == 'abyss-map' and options.k:
            align_reads_abyssmap(args[0], args[1], args[2], k=options.k, bamfile=options.bam)