#!/bin/bash

set -eux

help()
{
  echo "USAGE: submitjobs.sh CLUSTER_HEAD PATH_TO_MQSUB JOB_DIR JOB_FILE JOB_NAME JOB_MEM QUEUE [EMAIL]";
  echo "(PATH_TO_MQSUB in GSC: /opt/mqtools/bin/mqsub)"
  echo "EMAIL is the email address for sending job termination and abortion notices, optional."
  echo "This script is used together with configs/projects.cfg file, so changes made in this file should be reflected in the cfg file if necessary."
}

if [ $# -lt 6 ]
then
  help
  exit
fi

email=""

cluster=$1
mqsub=$2
dir=$3
file=$4
name=$5
mem=$6
queue=$7
if [ $# -gt 7 ]
then
    email=$8
fi

resource="-V -l mem_free=$mem -l mem_token=$mem -l h_vmem=$mem"
qsub="$resource -q $queue -o $dir -e $dir"

if [ "$queue" == 'thosts.q' ]
then
    qsub=$qsub" -P transfer"
fi

if [ "$email" != "" ]
then
    qsub=$qsub" -m as -M $email"
fi

# Check the job file and split if it contains more than 5000 jobs (GSC cluster job limit)
jobs=`wc -l $file |cut -f1 -d' '`

if [ "$cluster" == 'genesis' ]
then
    cmd="cd $dir && source /etc/profile.d/sge.sh && hostname && $mqsub --chdir $dir --qsub '$qsub' --name $name"
else
	if [ "$queue" != 'thosts.q' ]
	then
    	qsub=$qsub" -l hostname=q*"
	fi
    cmd="cd $dir && hostname && $mqsub --chdir $dir --qsub '$qsub' --name $name"
fi

host4=`hostname|cut -f1 -d'.'|awk '{ print substr( $0, 0, 4 ) }'`
if [ "$cluster" == 'genesis' -a "$host4" == 'node' ]
then
    cluster="login2"
fi

if [ $jobs -gt 5000 ]
then
  rm -f $file-split*
  split -l5000 $file $file-split
  for fs in `ls $file-split*`
    do
    cmd1=$cmd" --file $fs"
    echo $cmd1
    ssh $cluster \
	$cmd1
  done
else
  cmd=$cmd" --file $file"
  # Log into apollo and submit.
  if [ "$email" != "" ]
      then
      ssh $cluster \
	  $cmd
  else
      ssh $cluster \
	  $cmd
  fi
fi

exit
