from string import maketrans
import re, os, sys, grp

def set_attrs(obj, attrs):
    if attrs:
        for attr,value in attrs.iteritems():
            setattr(obj, attr, value)

def flatten(L):
    if type(L) != type([]): return [L]
    if L == []: return L
    return flatten(L[0]) + flatten(L[1:])

def reverse_complement(seq):
    complement = maketrans("agtcAGTC", "tcagTCAG")
    return seq[::-1].translate(complement)

def bamfile_contig_name(contig, library=None):
    cname = re.sub('[+-]', '', contig)
    cname = re.sub('[:,]', '_', cname)

    if library:
        cname = library + "_" + cname

    return cname

def is_homopolymer(seq):
    result = True

    if len(seq) < 2:
        result = False
    else:
        for i in range(1, len(seq)):
            if seq[0].upper() != seq[i].upper():
                result = False
                break

    return result

def get_refseq(fa, genome=None):
    from Bio import SeqIO

    sys.stderr.write("extracting reference sequences...")
    fasta_sequences = SeqIO.parse(open(fa), 'fasta')
    
    chrom_proper = None
    if genome:
	chrom_proper = ucsc_chroms(genome)

    refseq = {}
    for fasta in fasta_sequences:
        if re.match('^(chr|scaffold)', fasta.id, re.IGNORECASE):
            chrom = fasta.id
        else:
            chrom = 'chr' + fasta.id
	    
	if chrom_proper and chrom_proper.has_key(fasta.id):
	    chrom = chrom_proper[fasta.id]
	    
        refseq[chrom] = fasta
	
    sys.stderr.write("done\n")
    return refseq

def get_refseq_from_2bit(genome=None, two_bit_file=None):
    from two_bit import TwoBitFileCls
    if genome:
	package_dir = "/".join(os.path.abspath(__file__).split("/")[:-2])
	genome_dir = package_dir + '/annotations/' + genome
	two_bit = genome_dir + '/' + genome + '.2bit'
    elif two_bit_file:
	two_bit = two_bit_file
	
    refseq = None
    if os.path.exists(two_bit):
	refseq = TwoBitFileCls(genome_dir + '/' + genome + '.2bit')
    
    return refseq

def proper_chrom(chrom, genome):
    chrom_proper = ucsc_chroms(genome)
    
    if chrom_proper:
	if chrom_proper.has_key(chrom):
	    chrom = chrom_proper[chrom]
	elif chrom[:3].lower() == 'chr' and chrom_proper.has_key(chrom[3:]):
	    chrom = chrom_proper[chrom[3:]]
    
    elif not re.match('^(chr|scaffold)', chrom, re.IGNORECASE):
	chrom = 'chr' + chrom
	    
    return chrom

def get_splice_motifs(infile):
    motifs = {}
    if os.path.isfile(infile):
        for line in open(infile, 'r'):
            cols = line.rstrip("\n").split(' ')
            if len(cols) == 2:
                motifs[cols[0]] = cols[1]
            elif len(cols) == 1:
                motifs[cols[0]] = True

    return motifs

def get_chr_number(chrom):
    chr_pat = re.compile(r'chr(\S+)')
    
    m = chr_pat.search(chrom)
		
    if m and m.group:
	return m.group(1)
    else:
	return chrom
    
def is_integer(text):
    if re.match('^-?\d+$', text):
	return True
    else:
	return False
    
def ucsc_chroms(genome):
    package_dir = "/".join(os.path.abspath(sys.argv[0]).split("/")[:-2])
    conversion_file = package_dir + '/annotations/' + genome + "/ucsc_chr.txt"

    conversions = {}
    if os.path.exists(conversion_file):
	for line in open(conversion_file, 'r'):
	    chr_from, chr_to = line.rstrip('\n').split()
	    conversions[chr_from] = chr_to
	    
    return conversions

def output_log(version, argv, params_list, outdir):
    import datetime, socket
    logfile = outdir + '/LOG'
    log = open(logfile, 'a')
    log.write('[%s]\n' % (str(datetime.datetime.now())))
    log.write('command=%s %s %s\n' % (sys.executable, os.path.abspath(argv[0]), ' '.join(argv[1:])))
    log.write('host=%s\n' % (socket.gethostname()))
    log.write('version=%s\n' % (version))
    #log.write('%s %s %s\n' % (sys.executable, os.path.abspath(argv[0]), ' '.join(argv[1:])))
    for params in params_list:
	for param, value in params.iteritems():
	    log.write('%s=%s\n' % (param, value))
    log.write('\n')
    log.close()
    

def set_permissions(directory):
    gid = grp.getgrnam('trans-abyss').gr_gid
    for root, dirs, files in os.walk(directory):
        os.chown(root, -1, gid)
        os.chmod(root, 02775)
	
def seq_overlap(seq1, seq2):
    x = min(len(seq1), len(seq2))
    while x > 0:
	if seq1[-x:].upper() == seq2[:x].upper():
	    break
	x -= 1
	
    return x
