#!/bin/bash
#
# Usage: trans-abyss [-c] [-s] [-n] [-l] [-e] [-h] <-i input> <-1|-2|-3|-4|-5|-6|-7|-8|-9|-10|-11|-12>
#
set -eu

help()
{
    echo "trans-ABySS analysis pipeline v1.3"
    echo "Usage:"
    echo "trans-abyss [-c cluster_head] [-s start_library] [-n num_of_library] [-l library] [-e email] [-h|--help] [--version] <-i input> <-1|-2|-3|-4|-5|-6|-7|-8|-9> "
    echo ""
    echo "Options:"
    echo "-c cluster_head"
    echo "   run jobs on cluster (\"cluster_head\" is the head node of cluster)"
    echo "-s start_library"
    echo "   process libraries from \"start_library\" onwards (as specified in the input)"
    echo "-n num_of_library"
    echo "   process num_of_library libraries"
    echo "-l LIB"
    echo "   process exactly one library with the name \"LIB\""
    echo "-e EMAIL"
    echo "   for cluster jobs, send email notifications to \"EMAIL\" upon job completion, abortion, killing"
    echo "-h|--help"
    echo "   print this help message"
    echo "--version"
    echo "   print trans-ABySS version information"
    echo "-i input"
    echo "   input file name that specifies the libraries and ABySS source information"
    echo "-0 set-up"
    echo "   set up directories, get reads, filter/extend/merge contigs"
    echo "-1 reads-to-genome"
    echo "   copy reads-to-genome BAM file"
    echo "-2 reads-to-contigs"
    echo "   generate BWA reads-to-contigs alignment"
    echo "-3 xa2multi"
    echo "   convert BWA multi- alignments to 1-alignment-per-record"
    echo "-4 contigs-to-genome"
    echo "   align contigs to reference genome, BLAT, and filter alignments to create UCSC track"
    echo "-5 novel splicing"
    echo "   find novel transcript events"
    echo "-6 fusion"
    echo "   find candidate fusions on cluster"
    echo "-7 SNV, INDEL"
    echo "   find candidate SNVs and INDELs"
    echo "-8 expression"
    echo "   report gene expression levels"
    echo "-9 copy genesis"
    echo "   copy back library dir from genesis"
}

version()
{
    echo "trans-ABySS v1.3.2"
    echo "Written by Readman Chiu, Rong She, Mark Okada, Lucas Swanson, and Ka Ming Nip."
    echo ""
    echo "Copyright 2012 Canada's Michael Smith Genome Science Centre"
}

set_args()
{
    if [ "$2" != "" ]
    then
        args=$args" -"$1" "$2
    fi
}

#set values
input=""
args=""
cluster=""
startlib=""
num=""
lib=""
email=""
stage=0
genesis=" -genesis_dir validations/transcriptome"

#get options
#while getopts c:s:n:l:hv12345 opt
while [ $# -gt 0 ]
do
    case "$1" in
        "-c") shift; if [ $# -gt 0 ]; then cluster=$1; shift; else help; exit; fi;;
        "-s") shift; if [ $# -gt 0 ]; then startlib=$1; shift; else help; exit; fi;;
        "-n") shift; if [ $# -gt 0 ]; then num=$1; shift; else help; exit; fi;;
        "-l") shift; if [ $# -gt 0 ]; then lib=$1; shift; else help; exit; fi;;
        "-e") shift; if [ $# -gt 0 ]; then email=$1; shift; else help; exit; fi;;
        "-0") stage=0; shift;;
        "-1") stage=1; shift;;
        "-2") stage=2; shift;;
        "-3") stage=3; shift;;
        "-4") stage=4; shift;;
        "-5") stage=5; shift;;
        "-6") stage=6; shift;;
        "-7") stage=7; shift;;
        "-8") stage=8; shift;;
        "-9") stage=9; shift;;
        "-h") help; shift;;
        "--help") help; shift;;
        "--version") version; shift;;
        "-i") shift; if [ $# -gt -1 ]; then input=$1; echo "input: $input"; shift; else help; exit; fi;;
        *) help; exit;;
    esac
done

set_args "cluster" "$cluster"
set_args "start" "$startlib"
set_args "num" "$num"
set_args "lib" "$lib"
set_args "email" "$email"
#set_args "genesis" "$genesis"
if [ "$input" = "" ]
then
    help
    exit
fi

#run trans-ABySS
if [ -e $input ]
then
  if [ -f $input ]
  then
        case $stage in
            0) $TRANSABYSS_PATH/wrappers/setup.pl $input $args -make_dir;$TRANSABYSS_PATH/wrappers/setup.pl $input $args -fem;$TRANSABYSS_PATH/wrappers/setup.pl $input $args -get_reads;;
            1) $TRANSABYSS_PATH/wrappers/setup.pl $input $args -copy_bam;;
            2) $TRANSABYSS_PATH/wrappers/setup.pl $input $args -anchor_finalize;;
            3) $TRANSABYSS_PATH/wrappers/analyze.pl $input $args -xa2multi;;
            4) $TRANSABYSS_PATH/wrappers/analyze.pl $input $args -align_contigs -cluster $cluster;;
            5) $TRANSABYSS_PATH/wrappers/analyze.pl $input $args -novelty;;
            6) $TRANSABYSS_PATH/wrappers/analyze.pl $input $args -fusions;;
            7) $TRANSABYSS_PATH/wrappers/analyze.pl $input $args -snv_indel;;
            8) $TRANSABYSS_PATH/wrappers/analyze.pl $input $args -gene_cover gene_cover.txt -annotation_cover annot_cover.txt;;
            9) $TRANSABYSS_PATH/wrappers/setup.pl $input $args -copy_dir $genesis;;
        esac
  else
        echo "$input is not a regular file"
  fi
else
    echo "$input not found"
fi

